#!Spriteモジュール
#|テクスチャを管理するモジュール。テクスチャ画像を読み込み、それをセルに切り分けて管理します。

module Sprite
@texhash={} # key=any value=[ Texture, celldata , NMV]
	def dump
		p @texhash
	end
	def clear
		#$Sprite.clear
		#@ロードされているスプライトすべてを廃棄します。
		@texhash.each{|item|
			item[1][0].delete
			if item[1][2] then
				item[1][2].delete
			end
		}
		@texhash={}
	end
	def update
		#$Sprite.update
		#@ムービーテクスチャのアップデートをします。Game.updateで呼んでください。
		@texhash.each{|item|
			if item[1][2] then
				item[1][2].totexture(item[1][0])
			end
		}
	end
	def load (key,filename,loopflag=true,hasalpha=false)
		#$Sprite.load (key,String filename[,Boolean loopflag=true,Boolean hasalpha=false])
		#@スプライトをロードします。rubyのハッシュのキーには数値も文字列もシンボルも使えるので、状況で使い分けてください。
		#@これは画像を読み込む関数です。セルの切り分け等は他の関数で行います。デフォルトでは、全体をセル0とみなします。
		#@filenameにはNMVファイルも指定できます。この場合ムービーテクスチャになります。ムービーテクスチャの場合はループするかどうか、α値を持つムービーかどうかを指定できます。
		#@filenameに"*幅,高さ,色"(例："*800,600,0xFFFFFFFF")とすれば塗りつぶし画像を指定できます。

		if @texhash[key] then Sprite.delete(key) end
		if filename[0,1]=="*" then
			p=filename[1,1000].split(",")
			tex=Texture.new(p[0].to_i,p[1].to_i)
			hex=p[2]
			if hex[0,2]=="0x" then hex=hex[2,1000] end
			c=hex[0,2].hex.to_f*0x1000000+hex[2,1000].hex.to_f
			tex.fill(c)
		elsif filename[-3,3].downcase=="nmv" then
			nmv=NMV.new(filename)
			tex=Texture.new(nmv.getwidth,nmv.getheight)
			nmv.play(loopflag,hasalpha)
			nmv.totexturewait(tex)
		else
			tex=Texture.new(filename)
			nmv=nil
		end

		@texhash[key]=[ tex,[[0,0,tex.getwidth,tex.getheight]],nmv]
	end
	def text (key,x,y,font,text,*args)
		if @texhash[key] then
			temp=text % args
			font.put(@texhash[key][0],temp,x,y)
		else
			raise("存在しないテクスチャに描画しようとしました。")
		end
	end
	def setcelldata (key,celldata)
		#$Sprite.setcelldata(key,celldata)
		#@読んだスプライトをセルに切り分ける情報を設定する関数です。
		#@jsonで読み込んだ配列データを割り当ててください。
		if @texhash[key]==nil then raise("ロードしていないテクスチャを切り分けようとしました。") end
		@texhash[key][1]=celldata
	end
	def slice (key,w,h)
		#$Sprite.slice(key,Fixnum w,Fixnum h)
		#@読んだスプライトをセルに切り分ける関数です。幅と高さで格子状に切り分けます。
		if @texhash[key]==nil then raise("ロードしていないテクスチャを切り分けようとしました。") end
		celldata=[]
		tw=@texhash[key][0].getwidth
		th=@texhash[key][0].getheight
		cx=(tw/w).floor-1
		cy=(th/h).floor-1
		for j in 0..cy do
			for i in 0..cx do
				celldata.push [i*w,j*h,w,h]
			end
		end
		@texhash[key][1]=celldata
	end
	def gettexture(key)
		#$Sprite.gettexture(key) → Texture
		#@テクスチャオブジェクトを返します。
		return @texhash[key][0]
	end
	def delete (key)
		#$Sprite.delete(key)
		#@指定されたスプライトを削除します。
		@texhash[key][0].delete
		if texhash[key][2] then texhash[key][2].delete end
		@texhash[key]=nil
	end
	def copy (key_dest,key_src)
		#$Sprite.copy(key_dest,key_src)
		#@key_destで指定されたキーのスプライトを作成し、key_srcで指定されたキーのスプライトをコピーします。
		#@テクスチャを加工したいけど元のものも残しておきたい、というようなときに使えます。
		if @texhash[key_dest] then Sprite.delete(key_dest) end
		if @texhash[key_src][2] then raise("ムービーテクスチャはコピーできません。") end
		@texhash[key_dest]=[@texhash[key_src][0].clone,@texhash[key_src][1].clone,nil]
	end
	def draw(key,cell)
		#$Sprite.draw(key,Fixnum cell)
		#@キーとセルを指定し、描画します。Draw.fromとDraw.putを内部で呼び出します。
		if @texhash[key] then
			t=@texhash[key]
			Draw.from(t[1][cell][0],t[1][cell][1],t[1][cell][2],t[1][cell][3])
			Draw.put(t[0])
		else
			raise("存在しないテクスチャを描画しようとしました。")
		end
	end
	def get (key,cell)
		#$Sprite.get(key,Fixnum cell) → [Texture tex,Fixnum sx,Fixnum sy,Fixnum sw,Fixnum sh]
		#@キーとセルを指定し、Textureインスタンス、転送元x,y,幅、高さの情報が入った配列を得ます。見つからなかった場合はnilが戻ります。
		if @texhash[key] then
			t=@texhash[key]
			if t[1].size<=cell || cell<0 then return nil end
			return [t[0],t[1][cell][0],t[1][cell][1],t[1][cell][2],t[1][cell][3]]
		else
			return nil
		end
	end
	def isplaying (key)
		#$Sprite.isplaying(key)→Boolean
		#@キーで指定されたスプライトがムービーテクスチャである場合、再生中ならtrue、停止していればfalseを返します。
		#@ムービーテクスチャでない場合は常にfalseを返します。
		if @texhash[key] then
			if @texhash[key][2] then
				return @texhash[key][2].isplaying
			else
				return false
			end
		else
			raise("存在しないスプライトの再生状況を取得しようとしました。")
		end
	end
	def monotone(key,color)
		#$Sprite.monotone(key,color)
		#@スプライトを指定した色(0xAARRGGBB)でモノトーンにします。
		if @texhash[key] then
			@texhash[key][0].monotone(color)
		else
			raise("存在しないテクスチャに描画しようとしました。")
		end
	end
	def mask(key)
		#$Sprite.mask(key)
		#@スプライトをアルファチャンネル化（アルファ値だけが残った白色画像）にします。
		if @texhash[key] then
			@texhash[key][0].mask
		else
			raise("存在しないテクスチャに描画しようとしました。")
		end
	end
	module_function :dump,:clear,:update,:load,:slice,:delete,:copy,:get,:draw,:monotone,:mask,:gettexture,:text
end

#!Taskモジュール
#|シンプルに実装されたタスクシステムです。

module Task
	@taskhash={}
	@taskorder=[] # [priority,key]
	@task_add_buffer=[]
	@task_delete_buffer=[]
	@clear_flag=false
	def dump
		p @taskhash
	end
	def add (key,task,priority)
		#$Task.add(key,task,priority)
		#@タスクを登録します。ハッシュのキーとTaskObjectを継承したオブジェクトと優先度(小さいほうが先に実行される)を指定します。
		#@実際の登録作業は、Task.updateの開始時に行われます。Task.update中に実行した場合は、次のTask.updateの頭で登録されます。
		if ! task.kind_of?(TaskObject) then raise "TaskObject型でないものは登録できません。" end
		@task_add_buffer.push([key,task,priority])
	end
	def delete (key)
		#$Task.delete(key)
		#@タスクを削除します。
		#@実際の削除作業は、Task.updateの開始時に行われます。Task.update中に実行した場合は、次のTask.updateの頭で削除されます。
		if @taskhash[key] then
			@task_delete_buffer.push(key)
		end
	end
	def clear
		#$Task.clear
		#@すべてのタスクを削除します。
		#@実際の削除作業は、Task.updateの開始時に行われます。Task.update中に実行した場合は、次のTask.updateの頭で削除されます。
		@clear_flag=true
	end
	def update
		#$Task.update
		#@すべてのタスクを優先度順に実行します。Game.updateから呼び出してください。
		if @clear_flag then
			clear_i
		else
			delete_i
		end
		add_i
		@taskorder.each{|item|
			@taskhash[item[1]].update
		}
	end
	def finalize
		#$Task.finalize
		#@タスクすべてのdeleteを呼び出してタスクシステムを終了します。
		#@Game.finalizeから呼び出してください。
		@taskorder.each{|item|
			@taskhash[item[1]].delete
		}
	end
#以下private関数
private
	def add_i
		@task_add_buffer.each {|item|
			if @taskhash[item[0]] then raise "タスクのキーが重複しました。" end
			@taskhash[item[0]]=item[1]
			@taskorder.push([item[2],item[0]])
		}
		@taskorder.sort!
		@task_add_buffer=[]
	end
	def clear_i
		@taskorder.each{|item|
			@taskhash[item[1]].delete
		}
		@taskhash={}
		@taskorder=[]
		@clearflag=false
	end
	def delete_i
		@task_delete_buffer.each{|item|
			@taskhash[item].delete
			@taskhash[item]=nil
			@taskorder.delete_if{|item2|
				item2[1]==item
			}
		}
	end
	module_function :add,:delete,:clear,:update,:add_i,:delete_i,:clear_i,:finalize,:dump
end

#!TaskObjectクラス
#|Taskモジュールに登録するタスクを表現するクラスです。
#|これをそのまま使うのではなく、継承して使います。
class TaskObject
	def initialize
	#$TaskObject.new→TaskObject
	#@初期データは派生クラスをnewで作成する時に構築してください。
	end
	def update
	#$tobj.update
	#@Task.updateを呼び出すと、それぞれのTaskObjectのupdateメソッドが優先度順（小さいほうが先）に呼び出されます。
	end
	def delete
	#$tobj.delete
	#@タスクが削除されるときにそれぞれのdeleteメソッドが呼ばれます。
	end
end
